package vc.thinker.mqtt.proxyclient.actor.down;

import akka.actor.AbstractActor;
import akka.japi.pf.ReceiveBuilder;
import cn.hutool.core.thread.ThreadUtil;
import com.alibaba.fastjson.JSONArray;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import vc.thinker.absctacts.mqtt.entity.MqttResponse;
import vc.thinker.absctacts.protocolproxy.dto.IServiceExecuteResult;
import vc.thinker.absctacts.protocolproxy.dto.ServiceExecuteCmd;
import vc.thinker.absctacts.protocolproxy.protocol.IThirdPartyProtocol;
import vc.thinker.mqtt.proxyclient.entity.down.ServiceExecCmd;
import vc.thinker.mqtt.proxyclient.entity.up.ServiceExecResult;
import vc.thinker.mqtt.proxyclient.enums.ThirdPartyProxyTopicDefinition;
import vc.thinker.mqtt.proxyclient.service.ThirdPartyProxyClientService;

import javax.annotation.Resource;
import java.util.List;

/**
 * 新科云执行服务到此第三方代理
 *
 * @author HeTongHao
 * @since 2021/8/11 15:43
 */
@Slf4j
@Component
@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class ThirdPartyServiceExecuteActor extends AbstractActor {
    @Resource
    private ThirdPartyProxyClientService thirdPartyProxyClientService;
    @Resource
    private List<IThirdPartyProtocol> thirdPartyProtocols;

    public ThirdPartyServiceExecuteActor() {
        receive(ReceiveBuilder.match(MqttResponse.class, mqttResponse -> {
            String topic = mqttResponse.getTopic();
            String protocolType = ThirdPartyProxyTopicDefinition.resolveProtocolCodeByTopic(topic);
            for (IThirdPartyProtocol thirdPartyProtocol : thirdPartyProtocols) {
                if (protocolType.equals(thirdPartyProtocol.protocolType().getCode())) {
                    List<ServiceExecCmd> serviceExecuteCmds =
                            JSONArray.parseArray(mqttResponse.getMessage(), ServiceExecCmd.class);
                    // 发送到具体协议去执行
                    executeAndResponse(thirdPartyProtocol, serviceExecuteCmds);
                    break;
                }
            }
        }).build());
    }

    /**
     * 执行并响应结果
     *
     * @param thirdPartyProtocol 分发到的协议协议
     * @param serviceExecuteCmds 待执行的命令列表
     */
    private void executeAndResponse(IThirdPartyProtocol thirdPartyProtocol, List<ServiceExecCmd> serviceExecuteCmds) {
        serviceExecuteCmds.forEach(cmd ->
                ThreadUtil.execute(() -> {
                    // 执行->获取结果
                    IServiceExecuteResult result = thirdPartyProtocol.onExecutiveService(
                            new ServiceExecuteCmd(cmd.getRequestId(), cmd.getDeviceCode(), cmd.getServiceCode(), cmd.getParams())
                    );
                    // 结果响应
                    thirdPartyProxyClientService.executeResponse(new ServiceExecResult(thirdPartyProtocol.protocolType().getCode(), result));
                })
        );
    }
}
